@@ -1,75 +1,13 @@
-##' Return a list of studies that match given properties
-##'
-##' @title find_study
-##' @param exact Should exact matching be used? (logical, default
-##'     \code{FALSE})
-##' @param property The property to be searched on (character)
-##' @param value The property-value to be searched on (character)
-##' @param verbose Should the output include all metadata (logical
-##'     default \code{FALSE})
-##' @param ...  additional arguments to customize the API request (see
-##'     \code{\link{rotl}} package documentation).
-##' @seealso \code{\link{studies_properties}} which lists properties
-##'     against which the studies can be searched
-##' @export
-##' @examples
-##' \dontrun{
-##' study <- studies_find_studies(property="ot:studyId", value="pg_719")
-##' }
-
-studies_find_studies <- function(property=NULL, value=NULL, verbose=FALSE,
-                                 exact=FALSE, ...) {
-    res <- .studies_find_studies(property = property, value = value,
-                                 verbose = verbose, exact = exact, ...)
-    class(res) <- c("found_studies", class(res))
-    return(res)
-}
-
-##' Return a list of trees that match a given properties
-##'
-##' The list of possible values to be used as values for the argument
-##' \code{property} can be found using the function
-##' \code{\link{studies_properties}}.
-##'
-##' @title find trees
-##' @param property The property to be searched on (character)
-##' @param value The property-value to be searched on (character)
-##' @param verbose Should the output include all metadata? (logical,
-##'     default \code{FALSE})
-##' @param exact Should exact matching be used? (logical, default
-##'     \code{FALSE})
-##' @param ... additional arguments to customize the API request (see
-##'     \code{\link{rotl}} package documentation).
-##' @seealso \code{\link{studies_properties}} which lists properties
-##'     against which the studies can be searched
-##' @export
-##' @examples
-##' \dontrun{
-##' res <- studies_find_trees(property="ot:ottTaxonName", value="Garcinia")
-##' }
-
-studies_find_trees <- function(property=NULL, value=NULL, verbose=FALSE,
-                               exact=FALSE, ...) {
-    res <- .studies_find_trees(property = property, value = value,
-                               verbose = verbose, exact = exact, ...)
-    class(res) <- c("found_studies", class(res))
-    return(res)
-}
-
-##'@export
-print.found_studies <- function(x, ...){
-  
- cat(" List of Open Tree studies with", length(x[[1]]), "hits \n")
-}
 ##' Return the list of study properties that can be used to search
 ##' studies and trees used in the synthetic tree.
 ##'
 ##' The list returned has 2 elements \code{tree_properties} and
-##' \code{studies_properties}. Each of these elements lists theadditional arguments to customize the API request
-##' properties that can be used to search for trees and studies that
-##' are contributing to the synthetic tree.
+##' \code{studies_properties}. Each of these elements lists
+##' theadditional arguments to customize the API request properties
+##' that can be used to search for trees and studies that are
+##' contributing to the synthetic tree.
 ##'
-##' @title Studies properties
+##' @title Properties of the Studies
 ##' @param ...  additional arguments to customize the API request (see
 ##'     \code{\link{rotl}} package documentation).
 ##' @return A list of the study properties that can be used to find
@@ -88,9 +26,176 @@ studies_properties <- function(...) {
 }
 
 
+##' Return the identifiers of studies that match given properties
+##'
+##' @title Find a Study
+##' @param exact Should exact matching be used? (logical, default
+##'     \code{FALSE})
+##' @param property The property to be searched on (character)
+##' @param value The property value to be searched on (character)
+##' @param detailed If \code{TRUE} (default), the function will return
+##'     a data frame that summarizes information about the study (see
+##'     \sQuote{Value}). Otherwise, it only returns the study
+##'     identifiers.
+##' @param verbose Should the output include all metadata (logical
+##'     default \code{FALSE})
+##' @param ...  additional arguments to customize the API request (see
+##'     \code{\link{rotl}} package documentation).
+##' @return If \code{detailed=TRUE}, the function returns a data frame
+##'     listing the study id (\code{study_ids}), the number of trees
+##'     associated with this study (\code{n_trees}), the tree ids (at
+##'     most 5) associated with the studies (\code{tree_ids}), the
+##'     tree id that is a candidate for the synthetic tree if any
+##'     (\code{candidate}), the year of publication of the study
+##'     (\code{study_year}), the title of the publication for the
+##'     study (\code{title}), and the DOI (Digital Object Identifier)
+##'     for the study (\code{study_doi}).
+##'
+##'     If \code{detailed=FALSE}, the function returns a data frame
+##'     with a single column containing the study identifiers.
+##' @seealso \code{\link{studies_properties}} which lists properties
+##'     against which the studies can be
+##'     searched. \code{\link{list_trees}} that returns a list for all
+##'     tree ids associated with a study.
+##' @export
+##' @examples
+##' \dontrun{
+##' ## To match a study for which the identifier is already known
+##' one_study <- studies_find_studies(property="ot:studyId", value="pg_719")
+##' list_trees(one_study)
+##'
+##' ## To find studies pertaining to Mammals
+##' mammals <- studies_find_studies(property="ot:focalCladeOTTTaxonName",
+##'                                 value="mammalia")
+##' ## To extract the tree identifiers for each of the studies
+##' list_trees(mammals)
+##' ## ... or for a given study
+##' list_trees(mammals, "ot_308")
+##'
+##' ## Just the identifiers without other information about the studies
+##' mammals <- studies_find_studies(property="ot:focalCladeOTTTaxonName",
+##'                                 value="mammalia", detailed=FALSE)
+##' }
+studies_find_studies <- function(property=NULL, value=NULL, verbose=FALSE,
+                                 exact=FALSE, detailed = TRUE, ...) {
+    .res <- .studies_find_studies(property = property, value = value,
+                                 verbose = verbose, exact = exact, ...)
+    res <- vapply(.res[["matched_studies"]],
+                  function(x) x[["ot:studyId"]],
+                  character(1))
+    if (detailed) {
+        dat <- summarize_meta(res)
+    } else {
+        meta_raw <- .res
+        dat <- data.frame(study_ids = res, stringsAsFactors = FALSE)
+        attr(dat, "found_trees") <- paste("If you want to get a list of the",
+                                          "trees associated with the studies,",
+                                          "use", sQuote("detailed = TRUE"))
+        class(dat) <- c("study_ids", class(dat))
+        attr(dat, "metadata") <- meta_raw
+    }
+    class(dat) <- c("matched_studies", class(dat))
+    dat
+}
+
+##' @export
+print.study_ids <- function(x, ...) {
+    print(format(x), ...)
+}
+
+##' Return a list of studies for which trees match a given set of
+##' properties
+##'
+##' The list of possible values to be used as values for the argument
+##' \code{property} can be found using the function
+##' \code{\link{studies_properties}}.
+##'
+##' @title Find Trees
+##' @param property The property to be searched on (character)
+##' @param value The property-value to be searched on (character)
+##' @param verbose Should the output include all metadata? (logical,
+##'     default \code{FALSE})
+##' @param exact Should exact matching be used for the value?
+##'     (logical, default \code{FALSE})
+##' @param detailed Should a detailed report be provided? If
+##'     \code{TRUE} (default), the output will include metadata about
+##'     the study that include trees matching the property. Otherwise,
+##'     only information about the trees will be provided.
+##' @param ... additional arguments to customize the API request (see
+##'     \code{\link{rotl}} package documentation).
+##' @return A data frame that summarizes the trees found (and their
+##'     associated studies) for the requested criteria. If a study has
+##'     more than 5 trees, the \code{tree_ids} of the first ones will
+##'     be shown, followed by \code{...} to indicate that more are
+##'     present.
+##'
+##'     If \code{detailed=FALSE}, the data frame will include the
+##'     study ids of the study (\code{study_ids}), the number of trees
+##'     in this study that match the search criteria
+##'     (\code{n_matched_trees}), the tree ids that match the search
+##'     criteria (\code{match_tree_ids}).
+##'
+##'     If \code{detailed=TRUE}, in addition of the fields listed
+##'     above, the data frame will also contain the total number of
+##'     trees associated with the study (\code{n_trees}), all the tree
+##'     ids associated with the study (\code{tree_ids}), the tree id
+##'     that is a potential candidate for inclusion in the synthetic
+##'     tree (if any) (\code{candidate}), the year the study was
+##'     published (\code{study_year}), the title of the study
+##'     (\code{title}), the DOI for the study (\code{study_doi}).
+##'
+##' @seealso \code{\link{studies_properties}} which lists properties
+##'   the studies can be searched on. \code{\link{list_trees}} for
+##'   listing the trees that match the query.
+##' @export
+##' @importFrom stats setNames
+##' @examples
+##' \dontrun{
+##' res <- studies_find_trees(property="ot:ottTaxonName", value="Drosophilia",
+##'                           detailed=FALSE)
+##' ## summary of the trees and associated studies that match this criterion
+##' res
+##' ## With metadata about the studies (default)
+##' res <- studies_find_trees(property="ot:ottTaxonName", value="Drosophilia",
+##'                           detailed=TRUE)
+##' ## The list of trees for each study that match the search criteria
+##' list_trees(res)
+##' ## the trees for a given study
+##' list_trees(res, study_id = "pg_2769")
+##' }
+studies_find_trees <- function(property=NULL, value=NULL, verbose=FALSE,
+                               exact=FALSE, detailed = TRUE, ...) {
+    .res <- .studies_find_trees(property = property, value = value,
+                               verbose = verbose, exact = exact, ...)
+    study_ids <- vapply(.res[["matched_studies"]],
+                        function(x) x[["ot:studyId"]],
+                        character(1))
+    n_matched_trees <- vapply(.res[["matched_studies"]],
+                              function(x) length(x[["matched_trees"]]),
+                              numeric(1))
+    match_tree_ids <- lapply(.res[["matched_studies"]],
+                             function(x) {
+        sapply(x[["matched_trees"]],
+               function(y) y[["nexson_id"]])
+    })
+    tree_str <- vapply(match_tree_ids, limit_trees, character(1))
+    res <- data.frame(study_ids, n_matched_trees, match_tree_ids = tree_str,
+                      stringsAsFactors = FALSE)
+    if (detailed) {
+        meta <- summarize_meta(study_ids)
+        res <- merge(meta, res)
+        attr(res, "metadata") <- attr(meta, "metadata")
+    } else {
+        attr(res, "metadata") <- .res
+    }
+    attr(res, "found_trees") <- stats::setNames(match_tree_ids, study_ids)
+    class(res) <- c("matched_studies", class(res))
+    res
+}
 
 
-##' Returns a study for given ID
+
+##' Returns the trees associated with a given study
 ##'
 ##' If \code{file_format} is missing, the function returns an object
 ##' of the class \code{phylo} from the \code{\link[ape]{ape}} package
@@ -104,7 +209,7 @@ studies_properties <- function(...) {
 ##' with the same name already exists, it will be silently
 ##' overwritten.
 ##'
-##' @title Get Study
+##' @title Get all the trees associated with a particular study
 ##' @param study_id the study ID for the study of interest (character)
 ##' @param object_format the class of the object the query should
 ##'     return (either \code{phylo} or \code{nexml}). Ignored if
@@ -177,6 +282,12 @@ get_study <- function(study_id = NULL, object_format = c("phylo", "nexml"),
 ##'     (\code{newick} default, \code{nexus}, or \code{json}).
 ##' @param file the file name where the output of the function will be
 ##'     saved.
+##' @param deduplicate logical (default \code{TRUE}). If the tree
+##' returned by the study contains duplicated taxon names, should they
+##' be made unique? It is normally illegal for NEXUS/Newick tree
+##' strings to contain duplicated tip names. This is a workaround to
+##' circumvent this requirement. If \code{TRUE}, duplicated tip labels
+##' will be appended \code{_1}, \code{_2}, etc.
 ##' @param ...  additional arguments to customize the API request (see
 ##'     \code{\link{rotl}} package documentation).
 ##' @return if \code{file_format} is missing, an object of class
@@ -194,9 +305,9 @@ get_study <- function(study_id = NULL, object_format = c("phylo", "nexml"),
 ##'  head(get_study_tree(study_id="pg_1144", tree="tree2324", tip_label="ott_taxon_name")$tip.label)
 ##' }
 
-get_study_tree <- function(study_id=NULL, tree_id=NULL, object_format=c("phylo"),
+get_study_tree <- function(study_id = NULL, tree_id = NULL, object_format = c("phylo"),
                            tip_label = c("original_label", "ott_id", "ott_taxon_name"),
-                           file_format, file, ...) {
+                           file_format, file, deduplicate = TRUE, ...) {
 
     object_format <- match.arg(object_format)
     tip_label <- match.arg(tip_label)
@@ -224,7 +335,7 @@ get_study_tree <- function(study_id=NULL, tree_id=NULL, object_format=c("phylo")
         file_format <- "newick"
         res <- .get_study_tree(study_id = study_id, tree_id = tree_id,
                                format=file_format, tip_label = tip_label, ...)
-        res <- phylo_from_otl(res)
+        res <- phylo_from_otl(res, dedup = deduplicate)
     } else stop("Something is very wrong. Contact us.")
     res
 }
@@ -246,8 +357,11 @@ get_study_tree <- function(study_id=NULL, tree_id=NULL, object_format=c("phylo")
 ##'
 ##'   \item {candidate_for_synth} { The identifier of the tree(s) from
 ##'   the study used in the synthetic tree. This is a subset of the
-##'   result of \code{get_tree_ids}.}
+##'   result of \code{get_tree_ids}.
 ##'
+##'   \item {get_study_year} { The year of publication of the study. }
+##'
+##'   }
 ##' }
 ##'
 ##' @title Study Metadata
@@ -264,6 +378,7 @@ get_study_tree <- function(study_id=NULL, tree_id=NULL, object_format=c("phylo")
 ##' get_tree_ids(req)
 ##' candidate_for_synth(req)
 ##' get_publication(req)
+##' get_study_year(req)
 ##' }
 get_study_meta <- function(study_id, ...) {
     res <- .get_study_meta(study_id = study_id, ...)
@@ -274,47 +389,13 @@ get_study_meta <- function(study_id, ...) {
 
 ##' @export
 print.study_meta <- function(x, ...) {
-    cat("Metadata for OToL study ", attr(x, "study_id"), " . Contents:\n", sep="")
+    cat("Metadata for OToL study ", attr(x, "study_id"), ". Contents:\n", sep="")
     cat(paste0("  $nexml$", names(x$nexml)), sep="\n")
 }
 
-##' @export
-##' @rdname get_study_meta
-get_tree_ids <- function(sm) UseMethod("get_tree_ids")
-
-##' @export
-##' @rdname get_study_meta
-get_publication <- function(sm) UseMethod("get_publication")
-
-##' @export
-##' @rdname get_study_meta
-candidate_for_synth <- function(sm) UseMethod("candidate_for_synth")
-
-##' @export
-##' @rdname get_study_meta
-get_tree_ids.study_meta <- function(sm) {
-    ## only keep the number of the ID
-    st_id <- gsub("[^0-9]", "", sm[["nexml"]][["^ot:studyId"]])
-    unlist(sm[["nexml"]][["treesById"]][[paste0("trees", st_id)]][["^ot:treeElementOrder"]])
-}
-
-##' @export
-##' @rdname get_study_meta
-get_publication.study_meta <- function(sm) {
-    pub <- sm[["nexml"]][["^ot:studyPublicationReference"]]
-    attr(pub, "DOI") <- sm[["nexml"]][["^ot:studyPublication"]][["@href"]]
-    pub
-}
-
-##' @export
-##' @rdname get_study_meta
-candidate_for_synth.study_meta <- function(sm) {
-    unlist(sm[["nexml"]][["^ot:candidateTreeForSynthesis"]])
-}
-
 ##' Retrieve subtree from a specific tree in the Open Tree of Life data store
 ##'
-##' @title Study subtree
+##' @title Study Subtree
 ##' @param study_id the study identifier (character)
 ##' @param tree_id the tree identifier (character)
 ##' @param object_format the class of the object returned by the
